==> js-language-24.0.0-sources.jar: com/oracle/truffle/js/nodes/access/JSTargetableNode.java <==
/*
 * Copyright (c) 2018, 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.access;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.instrumentation.GenerateWrapper;
import com.oracle.truffle.api.instrumentation.ProbeNode;
import com.oracle.truffle.api.nodes.UnexpectedResultException;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.runtime.Errors;

@GenerateWrapper
public abstract class JSTargetableNode extends JavaScriptNode {

    public abstract Object executeWithTarget(VirtualFrame frame, Object target);

    /* Used evaluateTarget instead of executeTarget to avoid confusion of the DSL. */
    public Object evaluateTarget(VirtualFrame frame) {
        return getTarget().execute(frame);
    }

    public int executeIntWithTarget(VirtualFrame frame, Object target) throws UnexpectedResultException {
        Object o = executeWithTarget(frame, target);
        if (o instanceof Integer) {
            return (int) o;
        } else {
            CompilerDirectives.transferToInterpreterAndInvalidate();
            throw new UnexpectedResultException(o);
        }
    }

    public double executeDoubleWithTarget(VirtualFrame frame, Object target) throws UnexpectedResultException {
        Object o = executeWithTarget(frame, target);
        if (o instanceof Double) {
            return (double) o;
        } else if (o instanceof Integer) {
            return (int) o;
        } else {
            CompilerDirectives.transferToInterpreterAndInvalidate();
            throw new UnexpectedResultException(o);
        }
    }

    public JavaScriptNode getTarget() {
        if (this instanceof WrapperNode) {
            return ((JSTargetableNode) ((WrapperNode) this).getDelegateNode()).getTarget();
        }
        throw Errors.notImplemented("getTarget");
    }

    public static Object evaluateReceiver(JavaScriptNode targetNode, VirtualFrame frame, Object targetValue) {
        if (!(targetNode instanceof SuperPropertyReferenceNode)) {
            return targetValue;
        } else {
            return ((SuperPropertyReferenceNode) targetNode).getThisValue().execute(frame);
        }
    }

    @Override
    public WrapperNode createWrapper(ProbeNode probe) {
        return new JSTargetableNodeWrapper(this, probe);
    }
}

==> js-language-24.1.0-sources.jar: com/oracle/truffle/js/nodes/access/JSTargetableNode.java <==
/*
 * Copyright (c) 2018, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.js.nodes.access;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.frame.VirtualFrame;
import com.oracle.truffle.api.instrumentation.GenerateWrapper;
import com.oracle.truffle.api.instrumentation.ProbeNode;
import com.oracle.truffle.api.nodes.UnexpectedResultException;
import com.oracle.truffle.js.nodes.JavaScriptNode;
import com.oracle.truffle.js.runtime.Errors;

@GenerateWrapper
public abstract class JSTargetableNode extends JavaScriptNode {

    public abstract Object executeWithTarget(VirtualFrame frame, Object target);

    /* Used evaluateTarget instead of executeTarget to avoid confusion of the DSL. */
    public Object evaluateTarget(VirtualFrame frame) {
        return getTarget().execute(frame);
    }

    public int executeIntWithTarget(VirtualFrame frame, Object target) throws UnexpectedResultException {
        Object o = executeWithTarget(frame, target);
        if (o instanceof Integer) {
            return (int) o;
        } else {
            CompilerDirectives.transferToInterpreterAndInvalidate();
            throw new UnexpectedResultException(o);
        }
    }

    public double executeDoubleWithTarget(VirtualFrame frame, Object target) throws UnexpectedResultException {
        Object o = executeWithTarget(frame, target);
        if (o instanceof Double) {
            return (double) o;
        } else if (o instanceof Integer) {
            return (int) o;
        } else {
            CompilerDirectives.transferToInterpreterAndInvalidate();
            throw new UnexpectedResultException(o);
        }
    }

    public JavaScriptNode getTarget() {
        if (this instanceof WrapperNode) {
            return ((JSTargetableNode) ((WrapperNode) this).getDelegateNode()).getTarget();
        }
        throw Errors.notImplemented("getTarget");
    }

    @Override
    public String expressionToString() {
        if (this instanceof WrapperNode) {
            return ((JSTargetableNode) ((WrapperNode) this).getDelegateNode()).expressionToString();
        }
        return super.expressionToString();
    }

    public static Object evaluateReceiver(JavaScriptNode targetNode, VirtualFrame frame, Object targetValue) {
        if (!(targetNode instanceof SuperPropertyReferenceNode)) {
            return targetValue;
        } else {
            return ((SuperPropertyReferenceNode) targetNode).getThisValue().execute(frame);
        }
    }

    @Override
    public WrapperNode createWrapper(ProbeNode probe) {
        return new JSTargetableNodeWrapper(this, probe);
    }
}
